/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package tuwien.auto.eibxlator;

import tuwien.auto.eicl.util.EICLException;

/**
 * <p>
 * This class implements the KNX DPT 1 Application Layer service type BOOLEAN.
 * The ASDU is encoded in the lower nibble of the second APDU byte.
 * 
 * @author Bernhard Erb
 */

public class PointPDUXlator_Boolean extends PointPDUXlator {

    /**
     * Nr. 1.001 Data Point Type Switch; Values off on
     */
    public static final String[] DPT_SWITCH = { "1.001", "DPT_Switch", "Off", "On" };

    /**
     * Nr. 1.002 Data Point Type Boolean; Values false true;
     */
    public static final String[] DPT_BOOL = { "1.002", "DPT_Bool", "False", "True" };

    /**
     * Nr. 1.003 Data Point Type Enable; Values enable disable;
     */
    public static final String[] DPT_ENABLE = { "1.003", "DPT_Enable", "Disable", "Enalbe" };

    /**
     * Nr. 1.004 Data Point Type Ramp; Values no ramp, ramp
     */
    public static final String[] DPT_RAMP = { "1.004", "DPT_Ramp", "No ramp", "Ramp" };

    /**
     * Nr. 1.005 Data Point Type Alarm; Vaues no alarm, alarm
     */
    public static final String[] DPT_ALARM = { "1.005", "DPT_Alarm", "No alarm", "alarm" };

    /**
     * Nr. 1.006 Data Point Type Binary Value; Values low, high
     */
    public static final String[] DPT_BINARYVALUE = { "1.006", "DPT_BinaryValue", "Low", "High" };

    /**
     * Nr. 1.007 Data Point Type Step; Values decrease increase
     */
    public static final String[] DPT_STEP = { "1.007", "DPT_Step", "Decrease", "Increase" };

    /**
     * Nr. 1.008 Data Point Type UpDown; Values up down
     */
    public static final String[] DPT_UPDOWN = { "1.008", "DPT_UpDown", "Up", "Down" };

    /**
     * Nr. 1.009 Data Point Type OpenClose; Values open close
     */
    public static final String[] DPT_OPENCLOSE = { "1.009", "DPT_OpenClose", "Open", "Close" };

    /**
     * Nr. 1.010 Data Point Type Start; Values stop start
     */
    public static final String[] DPT_START = { "1.010", "DPT_Start", "Stop", "Start" };

    /**
     * Nr. 1.011 Data Point Type State; Values inactive active
     */
    public static final String[] DPT_STATE = { "1.011", "DPT_State", "Inactive", "Active" };

    /**
     * Nr. 1.012 Data Point Type Invert; Values inverted, not inverted
     */
    public static final String[] DPT_INVERT = { "1.012", "DPT_Invert", "Not inverted", "Inverted" };

    /**
     * Nr. 1.013 Data Point Type DimSendStyle;Values Start/Stop,Cyclic
     */
    public static final String[] DPT_DIMSENDSTYLE = { "1.013", "DPT_DimSendStyle", "Start/Stop",
            "Cyclicalli" };

    /**
     * Nr. 1.014 Data Point Type InputSource; Values Fixed,Calculated
     */
    public static final String[] DPT_INPUTSOURCE = { "1.014", "DPT_InputSource", "Fixed",
            "Calculated" };

    private static EICLMap eiclMap = null;

    /**
     * Default constructor
     */
    public PointPDUXlator_Boolean() {
        initMap();
    }

    /**
     * Inits the object with the passed key parameter.
     * 
     * @param _PointType_Minor_Key
     *            The minor type key.
     * @throws EICLException
     *             If the minor type key doesn't exist
     */
    public PointPDUXlator_Boolean(String _PointType_Minor_Key) throws EICLException {
        initMap();
        pdu = new short[1];
        pointType_Minor = eiclMap.get(_PointType_Minor_Key);

        if (pointType_Minor == null)
            throw new EICLException("Unknown service type");
    }

    /**
     * Method used for initialising the eiclMap. You don't have to use this
     */
    private void initMap() {
        if (eiclMap == null) {
            eiclMap = new EICLMap();
            eiclMap.put(DPT_SWITCH[0], DPT_SWITCH);
            eiclMap.put(DPT_BOOL[0], DPT_BOOL);
            eiclMap.put(DPT_ENABLE[0], DPT_ENABLE);
            eiclMap.put(DPT_RAMP[0], DPT_RAMP);
            eiclMap.put(DPT_ALARM[0], DPT_ALARM);
            eiclMap.put(DPT_BINARYVALUE[0], DPT_BINARYVALUE);
            eiclMap.put(DPT_STEP[0], DPT_STEP);
            eiclMap.put(DPT_UPDOWN[0], DPT_UPDOWN);
            eiclMap.put(DPT_OPENCLOSE[0], DPT_OPENCLOSE);
            eiclMap.put(DPT_START[0], DPT_START);
            eiclMap.put(DPT_STATE[0], DPT_STATE);
            eiclMap.put(DPT_INVERT[0], DPT_INVERT);
            eiclMap.put(DPT_DIMSENDSTYLE[0], DPT_DIMSENDSTYLE);
            eiclMap.put(DPT_INPUTSOURCE[0], DPT_INPUTSOURCE);
        }
    }

    /**
     * Sets the ASDU bit from a boolean.
     * 
     * @param _ASDU
     *            The boolean to be set.
     */
    public void setASDUfromBoolean(boolean _ASDU) {
        // XXX old code before: true = 0, false = 1 ??
    	if (_ASDU)
            pdu[0] = 0x0001;
        else
            pdu[0] = 0x0000;
    }

    /**
     * Accepts the ASDU values to the according minor type in string format.
     * 
     * @throws EICLException
     *             If the value can't be found.
     */
    public void setASDUfromString(String _ASDU) throws EICLException {
        if (eiclMap.getTypeValues(pointType_Minor[0])[0].equals(_ASDU)) { // false
            pdu[0] = 0x0000;
        } else if (eiclMap.getTypeValues(pointType_Minor[0])[1].equals(_ASDU)) { // true
            pdu[0] = 0x0001;
        } else
            throw new EICLException("Value not allowed with this Minor PointType");
    }

    /**
     * Parses the ASDU message, and returns the corresponding boolean.
     * 
     * @return The ASDU message boolean.
     */
    public boolean getASDUasBoolean() {
        boolean ret = true;
        if (((pdu[0]) & 0x0001) == 0)
            ret = false;
        return ret;
    }

    /**
     * Returns the corresponding minor type value.
     */
    public String getASDUasString() {
        if (getASDUasBoolean())
            return pointType_Minor[3];
        return pointType_Minor[2];
    }

    /**
     * Converts the input values in the requested APDU/ASDU format. The result
     * is the complete Application Layer encoding.
     * 
     * @return the asdu byte[]
     */
    public byte[] getAPDUByteArray() {
        short s = (short) (servicetype << 6);
        if (servicetype == A_GROUPVALUE_WRITE)
            s = (short) (s | pdu[0]);
        byte[] ret = new byte[2];
        ret[0] = (byte) ((s & 0xFF00) >> 8);
        ret[1] = (byte) (s & 0x00FF);
        return ret;
    }

    /**
     * Returns the set minor type.
     * 
     * @return the minor type
     */
    public String getPointTypeMinor() {
        return pointType_Minor[0];
    }

    /**
     * Get the type specific minor types
     * 
     * @return the minor types
     */
    public EICLMap getMinorTypes() {
        return eiclMap;
    }

}
