/*
 * Copyright (C) 2005 Bernhard Erb and contributors

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/*
 $Log: Dialog_Read.java,v $
 Revision 1.7  2006/08/11 11:58:37  woif
 Datapointtype
 Some features added in EICLdemo

 Revision 1.6  2005/12/28 09:59:21  gneug
 xlator lookup fixed

 */

package tuwien.auto.eicldemo.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JFrame;

import tuwien.auto.eibpoints.*;
import tuwien.auto.eibxlator.PointPDUXlator;
import tuwien.auto.eibxlator.PDUXlatorList;
import tuwien.auto.eicl.CEMI_Connection;
import tuwien.auto.eicl.UserLayer;
import tuwien.auto.eicl.struct.cemi.CEMI_L_DATA;
import tuwien.auto.eicl.util.*;
import tuwien.auto.eicl.struct.cemi.EIB_Address;

public class Dialog_Read extends javax.swing.JDialog {
    public static final long serialVersionUID = 1;

    private JComboBox cb_Points;

    private JButton b_Cancel;

    private JButton b_Read;

    private PointList pointList;
    
    private JLabel  l_major;
    
    private JLabel  l_majorvalue;
    
    private JLabel  l_minor;
    
    private JLabel  l_minorvalue;

    // private CEMI_Connection tunnel;
    
    private UserLayer userLayer;

    public Dialog_Read(JFrame frame, PointList _PointList, CEMI_Connection _Tunnel) {
        super(frame);
        pointList = _PointList;
        // tunnel = _Tunnel;
        userLayer = new UserLayer(_Tunnel); 
        initGUI();
    }

    private void initGUI() {
        this.getContentPane().setLayout(null);
        this.setTitle("Read datapoint");
        {
            cb_Points = new JComboBox();
            Point[] p = pointList.getAllPoints();
            for (int i = 0; i < p.length; i++) {
                cb_Points.addItem(p[i].getDeviceName());
            }
            this.getContentPane().add(cb_Points);
            cb_Points.setBounds(30, 10, 160, 20);
            cb_Points.addItemListener(new DialogRead_cb_Point_ItemListener(this));
        }
        {
            l_major = new JLabel();
            this.getContentPane().add(l_major);
            l_major.setBounds(30, 35, 80, 20);
            l_major.setText("Majortype");
        }
        {
            l_majorvalue = new JLabel();
            this.getContentPane().add(l_majorvalue);
            l_majorvalue.setBounds(110, 35, 160, 20);
        }
        {
            l_minor = new JLabel();
            this.getContentPane().add(l_minor);
            l_minor.setBounds(30, 60, 80, 20);
            l_minor.setText("Minortype");
        }
        {
            l_minorvalue = new JLabel();
            this.getContentPane().add(l_minorvalue);
            l_minorvalue.setBounds(110, 60, 160, 20);
        }
        {
            b_Read = new JButton();
            this.getContentPane().add(b_Read);
            b_Read.setText("Read");
            b_Read.setBounds(30, 90, 100, 20);
            b_Read.addActionListener(new DialogRead_Read_actionAdapter(this));
        }
        {
            b_Cancel = new JButton();
            this.getContentPane().add(b_Cancel);
            b_Cancel.setText("Cancel");
            b_Cancel.setBounds(160, 90, 100, 20);
            b_Cancel.addActionListener(new DialogRead_Cancel_actionAdapter(this));
        }
        try {
            setSize(300, 150);
            cb_Point_SelectionChanged(null);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void Cancel_actionPerformed(ActionEvent e) {
        this.dispose();
    }

    public void Read_actionPerformed(ActionEvent e) {
        try {
            Point p = pointList.getPoint(cb_Points.getSelectedItem().toString());
            PointPDUXlator xlator = PDUXlatorList.getPointPDUXlator(p.getMajorType(),
                    p.getMinorType());
            xlator.setServiceType(PointPDUXlator.A_GROUPVALUE_READ);
            CEMI_L_DATA data = new CEMI_L_DATA((byte) CEMI_L_DATA.MC_L_DATAREQ, new EIB_Address(),
                    p.getDeviceAddress()[0], xlator.getAPDUByteArray());
            // tunnel.sendFrame(data, CEMI_Connection.WAIT_FOR_CONFIRM);
            
            // XXX access on high level
            boolean result = false;
            result = userLayer.readBool(p.getDeviceAddress()[0]);
            // XXX
            
            synchronized (Main_Frame.textArea) {
                Main_Frame.textArea.append("Read request to " + p.getDeviceName() + " sent \n");
            }
            this.dispose();
        } catch (EICLException ex) {
            Main_Frame.showException(ex.getMessage());
        }
    }
    
    public void cb_Point_SelectionChanged(ItemEvent e) {
        try {
            if (cb_Points.getSelectedIndex() != -1) {
                Point p = pointList.getPoint(cb_Points.getSelectedItem().toString());
                l_majorvalue.setText(PDUXlatorList.getPointTypeMajor().getTypeDescription(p.getMajorType()));
                l_minorvalue.setText(PDUXlatorList.getPointPDUMinor(p.getMajorType()).get(p.getMinorType())[1]);
            }
        } catch (EICLException ex) {
            Main_Frame.showException(ex.getMessage());
        }
    }

}

class DialogRead_Cancel_actionAdapter implements ActionListener {
    Dialog_Read conni;

    public DialogRead_Cancel_actionAdapter(Dialog_Read adaptee) {
        conni = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        conni.Cancel_actionPerformed(e);
    }
}

class DialogRead_Read_actionAdapter implements ActionListener {
    Dialog_Read conni;

    public DialogRead_Read_actionAdapter(Dialog_Read adaptee) {
        conni = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        conni.Read_actionPerformed(e);
    }
}

class DialogRead_cb_Point_ItemListener implements ItemListener {
    Dialog_Read dp;

    public DialogRead_cb_Point_ItemListener(Dialog_Read _DialogRead) {
        dp = _DialogRead;
    }

    public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED)
            dp.cb_Point_SelectionChanged(e);

    }

}