package tuwien.auto.obix.mappinglogic;

import java.lang.reflect.Constructor;
import java.net.InetSocketAddress;

import tuwien.auto.eicl.*;
import tuwien.auto.eicl.struct.cemi.*;
import tuwien.auto.obix.objects.*;
import tuwien.auto.eibxlator.*;
import java.sql.Date;

public class MappingLogic implements IMappingLogic
{

	private EICLListener _listener;

	private UserLayer _ul;

	public MappingLogic()
	{
		establishConnection();
	}

	public IDPT readGroupAddress(IGroupAddress ga, String dpt) throws Exception
	{
		EIB_Address address = new EIB_Address((int) (ga.rawValue().getInt()), true);
		IDPT dp = new DPT();

		if (dpt.equals("DPT_B1"))
		{
			Boolean result = _ul.readBool(address);
			dp = new DPT_B1(result);
		}
		if (dpt.equals("DPT_U8"))
		{
			Integer result = _ul.readInt(address, false);
			dp = new DPT_U8(result);
		}
	
		return dp;
	}

	public void writeGroupAddress(IGroupAddress ga, IDPT dp) throws Exception
	{
		// decode destination address
		int rawAddress = (int) (ga.rawValue().getInt());
		EIB_Address address = new EIB_Address(rawAddress, true);
		String dpt = dp.getIs().toString();
		dpt = dpt.replace("/obix/def/", "");
		dpt = dpt.substring(0, dpt.length() - 1);

		if (dpt.equals("DPT_U8"))
			_ul.write(address, (int) (dp.getInt()), false);
		if (dpt.equals("DPT_B1"))
			_ul.write(address, dp.getBool());
	}


	public void initCache(IGroupAddress ga, int level)
	{
		// TODO Auto-generated method stub

	}

	public CacheEntry[] readCache(IGroupAddress ga, String dpt) throws Exception
	{
		// TODO: add real caching interface here

		CacheEntry[] ce = new CacheEntry[1];

		IDPT dp = readGroupAddress(ga, dpt);

		ce[0] = new CacheEntry(new Date(50), dp);

		return ce;
	}

	public int numberOfFreeEntries(IGroupAddress ga)
	{
		// TODO Auto-generated method stub
		return 0;
	}

	public void registerEventListener(IMLEventListener el)
	{
		_listener.addMLEventListener(el);
	}

	private void establishConnection()
	{
		
		InetSocketAddress isa = findIPRouter();
		try
		{
			_ul = new UserLayer(isa.getHostName(), isa.getPort());
		}
		catch (Exception e)
		{
			e.printStackTrace();
			System.out.println(e.getMessage());
			System.exit(1);
		}
	}

	protected void finalize()
	{
		closeConnection();
	}

	public void closeConnection()
	{
		try
		{
			_ul.disconnect();
		}
		catch (Exception e)
		{
			e.printStackTrace();
			System.out.println(e.getMessage());
			System.exit(1);
		}
	}

	/**
	 * Searches for an KNXnet/IP router.
	 * 
	 * @return Address of the first KNXnet/IP router available.
	 */
	private InetSocketAddress findIPRouter()
	{
		Discoverer d = new Discoverer();

		try
		{
			d.sendSearchRequest();

			synchronized (this)
			{
				this.wait(2000);
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
			System.exit(1);
		}

		InetSocketAddress[] isa = d.getSearchResponses();
		if (isa.length > 0)
		{
			for (int i = 0; i < isa.length; i++)
			{
				System.out.println(isa[i].getHostName());
			}
		}

		return isa[0];
	}
}
