package tuwien.auto.obix.server;

import java.net.*;
import java.io.*;

import tuwien.auto.obix.mappinglogic.*;
import tuwien.auto.obix.objectbroker.*;
import tuwien.auto.obix.objects.*;
import obix.*;
import obix.io.*;

public class ObixServer implements IObixServer
{
	private IObjectBroker _objectBroker;

	public ObixServer()
	{
		char[] inC = { 'a' };
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

		// initialize KNX specific contracts
		tuwien.auto.obix.objects.ContractInit.init();
		// IMappingLogic ml = new MappingLogicStub();
		MappingLogic ml = new MappingLogic();
		_objectBroker = new ObjectBroker(ml);

		// add initial objects to the database
		initObjects();

		try
		{
			new HTTPServer(80, this);
			while (inC[0] != 'q')
				in.read(inC);
			ml.closeConnection();
		}
		catch (IOException ioex)
		{
			ioex.printStackTrace();
			System.exit(1);
		}

		System.exit(0);
	}

	private void initObjects()
	{
		Obj o = new Obj("knx");
		o.setHref(new Uri("http://localhost/obix/knx/"));
		o.add(new Ref("process", new Uri("process/")));
		_objectBroker.addObj(o);
		o = new Obj("house");
		o.setHref(new Uri("http://localhost/obix/knx/process/"));
		o.add(new Ref("fe", new Uri("fe/")));
		_objectBroker.addObj(o);
		FunctionalEntity fe = new FunctionalEntity("foo");
		fe.setHref(new Uri("http://localhost/obix/knx/process/fe/"));
		fe.setDescription("foodesc");
		_objectBroker.addObj(fe);

		// test GCEs
		GCE_rpoint testbool = new GCE_rpoint("testbool");
		testbool.setHref(new Uri("http://localhost/obix/knx/process/gce/gce04/"));
		testbool.setDescription("BCU booltest");
		testbool.setValue(new DPT_B1(false));
		GroupAddress ga = (GroupAddress) (testbool.address());
		ga.setRawValue(512);
		_objectBroker.addObj(testbool);
		GCE_rpoint testint = new GCE_rpoint("testint");
		testint.setHref(new Uri("http://localhost/obix/knx/process/gce/gce05/"));
		testint.setDescription("BCU inttest");
		testint.setValue(new DPT_U8(100));
		GroupAddress ga2 = (GroupAddress) (testint.address());
		ga2.setRawValue(513);
		_objectBroker.addObj(testint);

		GCE_command boolc = new GCE_command("boolcmd");
		boolc.setHref(new Uri("http://localhost/obix/knx/process/gce/gce06/"));
		boolc.setDescription("BCU booltest write");
		boolc.setTransmitDPT("/obix/def/DPT_B1/");
		GroupAddress ga3 = (GroupAddress) (boolc.address());
		ga3.setRawValue(512);
		_objectBroker.addObj(boolc);

		GCE_command intc = new GCE_command("intcmd");
		intc.setHref(new Uri("http://localhost/obix/knx/process/gce/gce07/"));
		intc.setDescription("BCU inttest write");
		intc.setTransmitDPT("/obix/def/DPT_U8/");
		GroupAddress ga4 = (GroupAddress) (intc.address());
		ga4.setRawValue(513);
		_objectBroker.addObj(intc);

		// DPT references
		DPT_U8 dptU8 = new DPT_U8(0);
		dptU8.setHref(new Uri("http://localhost/obix/def/DPT_U8/"));
		_objectBroker.addObj(dptU8);
		DPT_B1 dptB1 = new DPT_B1(false);
		dptB1.setHref(new Uri("http://localhost/obix/def/DPT_B1/"));
		_objectBroker.addObj(dptB1);
		
	}

	public String readObj(URI href, String user)
	{
		System.out.println("read request on: " + href.getPath());
		Obj o = _objectBroker.pullObj(new Uri(href.toASCIIString()));
		o = ObjectFilter.filterObject(o, 0);
		return ObixEncoder.toString(o);
	}

	public String writeObj(URI href, String xmlStream)
	{
		Obj input = ObixDecoder.fromString(xmlStream);

		try
		{
			_objectBroker.pushObj(new Uri(href.toASCIIString()), input, false);
		}
		catch (Exception ex)
		{
			Err e = new Err("Error writing object to network" + ex.getMessage());
			return ObixEncoder.toString(e);
		}

		return xmlStream;
	}

	public String invokeOp(URI href, String xmlStream)
	{
		Obj input = ObixDecoder.fromString(xmlStream);

		try
		{
			_objectBroker.pushObj(new Uri(href.toASCIIString()), input, true);
		}
		catch (Exception ex)
		{
			Err e = new Err("Error invoking operation" + ex.getMessage());
			ex.printStackTrace();
			return ObixEncoder.toString(e);
		}

		return ObixEncoder.toString(new Obj());
	}

	/**
	 * Starts a new oBIX server.
	 * 
	 * @param args
	 *            No command line parameters available.
	 */
	public static void main(String[] args)
	{
		new ObixServer();
	}

}
