/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package tuwien.auto.eibxlator;

import java.util.*;
import tuwien.auto.eicl.util.EICLException;

/**
 * <p>
 * EICLMAP is a utility class for the eibxlator package. It is used as container
 * for the according major minor types in the respective pduxlator classes.
 * 
 * @author Bernhard Erb
 * @see tuwien.auto.eibxlator.PDUXlatorList
 * @see tuwien.auto.eibxlator.PointPDUXlator_2ByteFloat
 * @see tuwien.auto.eibxlator.PointPDUXlator_3BitControlled
 * @see tuwien.auto.eibxlator.PointPDUXlator_8BitUnsigned
 * @see tuwien.auto.eibxlator.PointPDUXlator_Boolean
 * @see tuwien.auto.eibxlator.PointPDUXlator
 * 
 */

public class EICLMap {

    Hashtable convert;

    /**
     * Initializes a empty EICLMAP object.
     */
    public EICLMap() {
        convert = new Hashtable();
    }

    /**
     * Adds a new type to the map.
     * 
     * @param _Key
     *            The type key
     * @param _Type
     *            An array describing the type.
     */
    public void put(String _Key, String[] _Type) {
        convert.put(_Key, _Type);
    }

    /**
     * Returns the according key to the passed type description.
     * 
     * @param _TypeDescription
     *            The search type description
     * @return The according type key.
     * @throws EICLException
     *             If no type is associated with this description
     */
    public final String getTypeKey(String _TypeDescription) throws EICLException {
        boolean goon = true;
        Enumeration enumeration = convert.elements();
        String[] help = null;
        while (enumeration.hasMoreElements() && goon) {
            help = (String[]) enumeration.nextElement();
            if (help[1].equals(_TypeDescription))
                goon = false;
        }
        if (goon)
            throw new EICLException("Type not found");
        return help[0];
    }

    /**
     * Get the description of the specified type.
     * 
     * @param _TypeKey
     *            The type key.
     * @return The type description of the specified type.
     * @throws EICLException
     *             If type with the specified key can be found.
     */
    public final String getTypeDescription(String _TypeKey) throws EICLException {
        if (convert.get(_TypeKey) != null)
            return new String(((String[]) convert.get(_TypeKey))[1]);
        throw new EICLException("Type Description not found");
    }

    /**
     * Returns an array containing the description of all types known to this
     * map object.
     * 
     * @return The array with all type descriptions.
     */
    public String[] getAllTypeDescription() {
        String[] ret = new String[convert.size()];
        Enumeration enumeration = convert.elements();
        int i = 0;
        while (enumeration.hasMoreElements()) {
            ret[i] = ((String[]) enumeration.nextElement())[1];
            i++;
        }
        return ret;
    }

    /**
     * Returns the according values to the given type key.
     * 
     * @param _TypeKey
     *            The type key
     * @return The type values.
     * @throws EICLException
     *             if the key doesn't exist
     */
    public String[] getTypeValues(String _TypeKey) throws EICLException {
        String[] help = (String[]) convert.get(_TypeKey);
        if (help == null)
            throw new EICLException("Type not found");

        String[] ret = new String[help.length - 2];
        for (int i = 2; i < help.length; i++) {
            ret[i - 2] = help[i];
        }
        return ret;

    }

    /**
     * Returns the whole type array.
     * 
     * @param key
     *            The type key.
     * @return Returns an array describing this type.
     */
    public String[] get(String key) {
        return (String[]) convert.get(key);
    }

}
