/*
 * Copyright (C) 2005 Bernhard Erb
 * Copyright 2005-2006 Contributors

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/*
 $Log: PDUXlatorList.java,v $
 Revision 1.8  2006/01/11 19:05:10  gneug
 PointPDUXlator_String added

 */

package tuwien.auto.eibxlator;

import tuwien.auto.eicl.util.*;

/**
 * <p>
 * The PDUXlatorList is a utility class of the eibxlator package. It is useful
 * to lookup for DPT major types, and accessing their corresponding minor types.
 * Using the getPointPDUXlator() method an instance of the respective eibxlator
 * is returned. Note that only the DPT types implemented in this version are
 * contained and therefore accessible in this list.
 * 
 * @author bernhard erb
 * @see tuwien.auto.eibxlator.PDUXlatorList
 * @see tuwien.auto.eibxlator.PointPDUXlator_2ByteFloat
 * @see tuwien.auto.eibxlator.PointPDUXlator_3BitControlled
 * @see tuwien.auto.eibxlator.PointPDUXlator_8BitUnsigned
 * @see tuwien.auto.eibxlator.PointPDUXlator_Boolean
 * @see tuwien.auto.eibxlator.PointPDUXlator_String
 * @see tuwien.auto.eibxlator.PointPDUXlator
 */

public class PDUXlatorList {
    // Parameter 2 indicates the corresponding class implementation.
    /**
     * DPT type Boolean
     */
    public static final String[] TYPE_BOOLEAN = { "0x01", "Boolean (DPT 1)",
            "tuwien.auto.eibxlator.PointPDUXlator_Boolean" };

    /**
     * DPT type Dimming
     */
    public static final String[] TYPE_DIMMING = { "0x02", "Dimming (EIS 2)" };

    /**
     * DPT type 3Bit Controlled
     */
    public static final String[] TYPE_3BIT_CONTROLLED = { "0x03", "3 BIT controlled (DPT 3)",
            "tuwien.auto.eibxlator.PointPDUXlator_3BitControlled" };

    /**
     * DPT type Date
     */
    public static final String[] TYPE_DATE = { "0x04", "Date" };

    /**
     * DPT type Float
     */
    public static final String[] TYPE_2BYTE_FLOAT = { "0x05", "2 Byte Float Value (EIS 5)",
            "tuwien.auto.eibxlator.PointPDUXlator_2ByteFloat" };

    /**
     * DPT type 8 Bit unsigned
     */
    public static final String[] TYPE_8BIT_UNSIGNED = { "0x06", "8 BIT unsigned (DPT 6)",
            "tuwien.auto.eibxlator.PointPDUXlator_8BitUnsigned" };

    /**
     * DPT type Drive control
     */
    public static final String[] TYPE_DRIVE_CONTROL = { "0x07", "Drive Control (EIS 7)" };

    /**
     * DPT type Priority
     */
    public static final String[] TYPE_PRIORITY = { "0x08", "Priority" };

    /**
     * DPT type Float Value
     */
    public static final String[] TYPE_FLOAT_VALUE = { "0x09", "Float value" };

    /**
     * DPT type 16 Bit counter
     */
    public static final String[] TYPE_16BIT_COUNTER = { "0x10", "16 Bit Counter" };

    /**
     * DPT type 32 Bit counter
     */
    public static final String[] TYPE_32BIT_COUNTER = { "0x11", "32 Bit Counter" }; // not

    // implemented

    /**
     * DPT type Access
     */
    public static final String[] TYPE_ACCESS = { "0x12", "Access" };

    /**
     * DPT type EIB Ascii
     */
    public static final String[] TYPE_EIBASCII_CHAR = { "0x13", "Eib ASCII" };

    /**
     * DPT type 8 Bit counter
     */
    public static final String[] TYPE_8BIT_COUNTER = { "0x14", "8 Bit Counter" };

    /**
     * DPT type Character string
     */
    public static final String[] TYPE_CHARACTER_STRING = { "0x15", "String (DPT 16)",
            "tuwien.auto.eibxlator.PointPDUXlator_String" };

    private static EICLMap eiclMap = null;

    private static void initEICLMap() {
        eiclMap = new EICLMap(); // fillHashtable
        // eiclMap.put(TYPE_16BIT_COUNTER[0],TYPE_16BIT_COUNTER);
        // eiclMap.put(TYPE_32BIT_COUNTER[0],TYPE_32BIT_COUNTER);
        // eiclMap.put(TYPE_8BIT_COUNTER[0],TYPE_8BIT_COUNTER);
        // eiclMap.put(TYPE_ACCESS[0],TYPE_ACCESS);
        eiclMap.put(TYPE_CHARACTER_STRING[0], TYPE_CHARACTER_STRING);
        // eiclMap.put(TYPE_DATE[0],TYPE_DATE);
        eiclMap.put(TYPE_2BYTE_FLOAT[0], TYPE_2BYTE_FLOAT);
        eiclMap.put(TYPE_DIMMING[0], TYPE_DIMMING);
        eiclMap.put(TYPE_3BIT_CONTROLLED[0], TYPE_3BIT_CONTROLLED);
        eiclMap.put(TYPE_DRIVE_CONTROL[0], TYPE_DRIVE_CONTROL);
        // eiclMap.put(TYPE_EIBASCII_CHAR[0],TYPE_EIBASCII_CHAR);
        // eiclMap.put(TYPE_FLOAT_VALUE[0],TYPE_FLOAT_VALUE);
        // eiclMap.put(TYPE_PRIORITY[0],TYPE_PRIORITY);
        eiclMap.put(TYPE_8BIT_UNSIGNED[0], TYPE_8BIT_UNSIGNED);
        eiclMap.put(TYPE_BOOLEAN[0], TYPE_BOOLEAN);
        // eiclMap.put(TYPE_TIME[0],TYPE_TIME);
        // eiclMap.put(TYPE_VALUE[0],TYPE_VALUE);
    }

    /**
     * Returns the map of all implemented Major types
     * 
     * @return The major type map.
     */
    public static EICLMap getPointTypeMajor() {
        if (eiclMap == null)
            initEICLMap();
        return eiclMap;
    }

    /**
     * Returns an PDU translator instance for the requested major and minor type .
     * 
     * @param _MajorTypeKey
     *            The requested major type
     * @param _MinorTypeKey
     *            The requested minor type
     * @return An instance of the to the major and minor type corresponding
     *         translator
     * @throws EICLException
     *             If either the major or minor type can not be found.
     */
    public static PointPDUXlator getPointPDUXlator(String _MajorTypeKey, String _MinorTypeKey)
            throws EICLException {
        if (eiclMap == null)
            initEICLMap();
        String[] val = eiclMap.getTypeValues(_MajorTypeKey);
        PointPDUXlator xlator = null;
        try {
            Class[] c = new Class[] { String.class };
            xlator = (PointPDUXlator) Class.forName(val[0]).getConstructor(c).newInstance(
                    new Object[] { _MinorTypeKey });
        } catch (Exception ex) {
            throw new EICLException("Unimplemented DPT");
        }
        return xlator;
    }

    /**
     * Returns a list containing all known minor types of the requested major
     * type.
     * 
     * @param _MajorKey
     *            The major type
     * @return The map containing the corresponding minor types.
     * @throws EICLException
     *             If the major type can't be found.
     */
    public static EICLMap getPointPDUMinor(String _MajorKey) throws EICLException {
        if (eiclMap == null)
            initEICLMap();
        EICLMap ret = null;
        String[] val = eiclMap.getTypeValues(_MajorKey);
        try {
            PointPDUXlator xlator = (PointPDUXlator) Class.forName(val[0]).newInstance();
            ret = xlator.getMinorTypes();
        } catch (ClassNotFoundException ex) {
            throw new EICLException(ex.getMessage());
        } catch (ExceptionInInitializerError ex) {
            throw new EICLException(ex.getMessage());
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new EICLException(ex.getMessage());
        }
        return ret;
    }

}
