/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package tuwien.auto.eibxlator;

import tuwien.auto.eicl.util.*;

/**
 * <p>
 * This abstract class, is the base for every Application Layer encoding,
 * decoding (LA_Modem) class.
 * 
 * @author Bernhard Erb
 */
public abstract class PointPDUXlator {
    /**
     * Application Layer group value read
     */
    public static final short A_GROUPVALUE_READ = 0x00;

    /**
     * Application Layer group value response
     */
    public static final short A_GROUPVALUE_RESPONSE = 0x01;

    /**
     * Application Layer group value write
     */
    public static final short A_GROUPVALUE_WRITE = 0x02;

    short servicetype;

    short[] pdu;

    String[] pointType_Minor;

    /**
     * Dummy constructor only used for initialising the eiclMap. Don't use this
     */
    public PointPDUXlator() {
    };

    /**
     * Returns the ASDU as string
     * 
     * @return the ASDU string
     */
    public abstract String getASDUasString();

    /**
     * Converts the input values in the requested APDU/ASDU format. The result
     * is the complete Application Layer encoding.
     * 
     * @return the asdu byte[]
     */
    public abstract byte[] getAPDUByteArray();

    /**
     * Get the message service type (GROUP_VALUE_READ,GROUP_VALUE_WRITE....)
     * 
     * @return the service type
     */
    public short getServiceType() {
        return servicetype;
    }

    /**
     * Sets the message service type (GROUPVALUE_READ,GROUPVALUE_WRITE)
     * 
     * @param _ServiceType
     *            the service type
     * @throws EICLException
     */
    public final void setServiceType(short _ServiceType) throws EICLException {
        if (servicetype < A_GROUPVALUE_READ && servicetype > A_GROUPVALUE_WRITE)
            throw new EICLException("Unknown servicetype");
        servicetype = _ServiceType;
    }

    /**
     * Sets the ASDU value as string. Use this method for encoding.
     * 
     * @param _ASDU
     *            the ASDU value
     * @throws EICLException
     */
    public abstract void setASDUfromString(String _ASDU) throws EICLException;

    /**
     * Returns the set minor type.
     * 
     * @return the minor type
     */
    public abstract String getPointTypeMinor();

    /**
     * sets the APDU/ASDU as byte array, use getPdu to retrieve the value. Use
     * this method for decoding.
     * 
     * @param _APDU
     *            APDU/ASDU byte array
     * @throws EICLException
     */
    public final void setAPDUByteArray(byte[] _APDU) throws EICLException {
        if (_APDU.length < 2)
            throw new EICLException("APDU[] must have at least 2 bytes");
        short help = (short) (_APDU[0] << 8 | _APDU[1]);
        servicetype = (byte) (help & 0x03C0);
        if (servicetype < A_GROUPVALUE_READ && servicetype > A_GROUPVALUE_WRITE)
            throw new EICLException("Uknown servicetype");
        if (servicetype != A_GROUPVALUE_READ) {
            if (_APDU.length == 2) {
                pdu = new short[1];
                pdu[0] = (short) (_APDU[1] & 0x3F);
            } else {
                pdu = new short[_APDU.length - 2];
                for (int i = 2; i < _APDU.length; i++) {
                    pdu[i - 2] = _APDU[i];
                }
            }
        }
    }

    /**
     * Get the type specific minor types
     * 
     * @return the minor types
     */
    public abstract EICLMap getMinorTypes();
}
