/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package tuwien.auto.eicl.event;

import java.util.EventListener;

/**
 * <p>
 * With an implementation of this interface you can register a client to the
 * CEMI_Connection object. This event listener is invoked on every incoming
 * cEMI_L_DATA packet and on disconnect.
 * 
 * @see tuwien.auto.eicl.event.FrameEvent
 * @see tuwien.auto.eicl.event.DisconnectEvent
 * @see tuwien.auto.eicl.CEMI_Connection
 * @author Bernhard Erb
 */

public interface EICLEventListener extends EventListener {

    /**
     * A new frame has been received. The frame can be accessed through the
     * EICLEvent object.
     * 
     * @param e
     *            the event object
     */
    public void newFrameReceived(FrameEvent e);

    /**
     * The connection has been closed.
     * 
     * @param e
     *            The reason for connection ending.
     */
    public void serverDisconnected(DisconnectEvent e);
}
