/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package tuwien.auto.eicl.struct.eibnetip;

import java.io.*;

import tuwien.auto.eicl.struct.eibnetip.util.CRI_CRD;
import tuwien.auto.eicl.struct.eibnetip.util.EIBNETIP_Constants;
import tuwien.auto.eicl.struct.eibnetip.util.HPAI;
import tuwien.auto.eicl.util.*;

/**
 * <p>
 * This class creates an EIBNET/IP Connection Request. A Connection Request is
 * sent to server to open a point to point connection (Tunnelling or
 * Management). The server replies then immediately with a Connection Response.
 * Note that in this api version only tunnelling on link layer is supported.
 * 
 * @see tuwien.auto.eicl.struct.eibnetip.Connect_Response
 * @see tuwien.auto.eicl.CEMI_Connection
 * @author Bernhard Erb
 */

public class Connect_Request {
    private HPAI endpoint;

    private CRI_CRD cri;

    /**
     * Creates a new Connect_Request object. Only tunnelling on link layer is
     * supported in this version, all other connection codes cause an exception.
     * 
     * @param _Connectiontypecode
     *            the connection type code
     * @param _LocalPort
     *            the local server port
     * @param _Options
     *            The options processed in the HPAI structure after the byte 2.
     * @throws EICLException
     *             If the connection type code is not TUNNEL_CONNECTION this
     *             Exception is thrown
     */
    public Connect_Request(short _Connectiontypecode, short[] _Options, int _LocalPort)
            throws EICLException {
        endpoint = new HPAI(_LocalPort);
        cri = new CRI_CRD(_Connectiontypecode, _Options);
    }

    /**
     * The byte representation of this message.
     * 
     * @return byte The message in byte array representation.
     * @throws EICLException
     *             The IOException is forwarded as EICLException
     */

    public byte[] toByteArray() throws EICLException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        EIBnetIPPacket header = new EIBnetIPPacket(EIBNETIP_Constants.CONNECT_REQUEST,
                EIBNETIP_Constants.HEADER_SIZE_10 + endpoint.getStructLength() * 2
                        + cri.getStructLength());
        try {
            baos.write(header.toByteArray());
            baos.write(endpoint.toByteArray());
            baos.write(endpoint.toByteArray());
            baos.write(cri.toByteArray());
        } catch (IOException ex) {
            throw new EICLException(ex.getMessage());
        }
        return baos.toByteArray();
    }
}
