/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package tuwien.auto.eicl.struct.eibnetip;

import java.io.*;

import tuwien.auto.eicl.struct.eibnetip.util.EIBNETIP_Constants;
import tuwien.auto.eicl.struct.eibnetip.util.HPAI;
import tuwien.auto.eicl.util.*;

/**
 * <p>
 * This class represents an EIBnet/IP Connectionstate Request message. This
 * message is sent in an interval of 60 seconds to the server, acting as heart
 * beating message. If the message was not confirmed by a Connectionstate
 * Response, the request is replied two times. If even this attempts were not
 * successful the connection is closed.
 * 
 * @see tuwien.auto.eicl.struct.eibnetip.Connectionstate_Response
 * @see tuwien.auto.eicl.CEMI_Connection
 * @author Bernhard Erb
 */

public class Connectionstate_Request {

    private short channelid;

    private short reserved;

    private HPAI endpoint;

    /**
     * Creates a new Connectionstate_Request message.
     * 
     * @param _Channelid
     *            The current connection channelid.
     * @param _LocalPort
     *            The clients local data end point port.
     * @throws EICLException
     *             Is forwarded from HPAI class
     * 
     * @see tuwien.auto.eicl.struct.eibnetip.util.HPAI
     */
    public Connectionstate_Request(short _Channelid, int _LocalPort) throws EICLException {
        channelid = _Channelid;
        reserved = 0;
        endpoint = new HPAI(_LocalPort);
    }

    /**
     * Get the message in a byte array representation.
     * 
     * @return The message in byte array.
     * @throws EICLException
     *             If an IOException is thrown.
     */

    public byte[] getByteArray() throws EICLException {
        EIBnetIPPacket header = new EIBnetIPPacket(EIBNETIP_Constants.CONNECTIONSTATE_REQUEST,
                EIBNETIP_Constants.HEADER_SIZE_10 + 2 + endpoint.getStructLength());
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try {
            baos.write(header.toByteArray());
            baos.write((byte) channelid & 0x00FF);
            baos.write((byte) reserved & 0x00FF);
            baos.write(endpoint.toByteArray());
        } catch (IOException ex) {
            throw new EICLException(ex.getMessage());
        }

        return baos.toByteArray();

    }

}
