/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package tuwien.auto.eicl.struct.eibnetip;

import tuwien.auto.eicl.struct.eibnetip.util.Device_DIB;
import tuwien.auto.eicl.struct.eibnetip.util.SUPPFAM_DIB;
import tuwien.auto.eicl.util.*;

/**
 * <p>
 * This class parses a byte stream and initiates a description response. The
 * second device description information block is optional, and thus not
 * implemented. A description response is sent in answer to a description
 * request by a server.
 * 
 * @author Bernhard Erb
 * @see tuwien.auto.eicl.struct.eibnetip.Description_Request
 */

public class Description_Response {
    private Device_DIB device;

    private SUPPFAM_DIB suppfam;

    // private Device_DIB other; optional so not implemented

    /**
     * Creates an empty Description Response instance. (dummy constructor)
     */
    public Description_Response() {
        // dummy constructor
    }

    /**
     * Creates a description response by parsing a byte array. Pass all bytes
     * after the EIBnet/IP header to this constructor.
     * 
     * @param _Buffer
     *            The byte array starting after the EIBnet/IP header.
     * @throws EICLException
     *             Forward the EICLException thrown by the Device_DIB and
     *             SUPPFAM_DIB class
     */
    public Description_Response(byte[] _Buffer) throws EICLException {
        device = new Device_DIB(_Buffer);
        byte[] help = new byte[_Buffer.length - device.getStructlength()];
        for (int i = device.getStructlength(); i < _Buffer.length; i++) {
            help[i - device.getStructlength()] = _Buffer[i];
        }
        suppfam = new SUPPFAM_DIB(help);
    }

    /**
     * Get the device description information block contained in this message.
     * 
     * @return The device description information block.
     */
    public Device_DIB getDevice() {
        return device;
    }

    /**
     * Get the supported families description information block.
     * 
     * @return The supported families description information block.
     */
    public SUPPFAM_DIB getSupportedFamilies() {
        return suppfam;
    }
}
