/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package tuwien.auto.eicl.struct.eibnetip;

import java.io.*;

import tuwien.auto.eicl.struct.eibnetip.util.EIBNETIP_Constants;
import tuwien.auto.eicl.struct.eibnetip.util.HPAI;
import tuwien.auto.eicl.util.*;

/**
 * Created on 10.04.2005
 * <p>
 * This class is the implementation of the EIBnet/IP Search Request. A search
 * request is sent multicast over the local IP network to port 3671. All
 * receiving EIBnet/IP server replies with a Search Response. This process can
 * be automated using the Discoverer class of the tuwien.auto.eicl package.
 * 
 * @see tuwien.auto.eicl.struct.eibnetip.Search_Response
 * @see tuwien.auto.eicl.Discoverer
 * @author Bernhard Erb
 */

public class Search_Request {

    private HPAI endpoint;

    /**
     * Initializes a new search request object for sending.
     * 
     * @param _PortNumber
     *            The local port number.
     * @throws EICLException
     *             Forwards the Exception thrown by the constructor of the HPAI
     *             object.
     */
    public Search_Request(int _PortNumber) throws EICLException {
        endpoint = new HPAI(_PortNumber);
    }

    /**
     * Returns the byte array representation of this packet
     * 
     * @return The byte array message.
     * @throws EICLException
     */
    public byte[] toByteArray() throws EICLException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try {
            EIBnetIPPacket header = new EIBnetIPPacket(EIBNETIP_Constants.SEARCH_REQUEST,
                    (char) (EIBNETIP_Constants.HEADER_SIZE_10 + endpoint.getStructLength()));
            baos.write(header.toByteArray());
            baos.write(endpoint.toByteArray());

        } catch (IOException ex) {
            throw new EICLException(ex.getMessage());
        }
        return baos.toByteArray();
    }

    /**
     * Returns the end point HPAI
     * 
     * @return The end point HPAI
     */
    public HPAI getEndPoint() {
        return endpoint;
    }
}
