/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package tuwien.auto.eicl.struct.eibnetip;

import tuwien.auto.eicl.struct.eibnetip.util.HPAI;
import tuwien.auto.eicl.util.*;

/**
 * <p>
 * This class implements the EIBnet/IP search response message. This message is
 * sent in replie to a Search Response.
 * 
 * @see tuwien.auto.eicl.struct.eibnetip.Search_Request
 * @author Bernhard Erb
 */
public class Search_Response {
    private HPAI discovered_endpoint;

    /**
     * Initializes the object by parasing a byte array. Throws EICLException if
     * the parsing didn't succeed.
     * 
     * @param _Body
     *            The EIBnet/IP message body (after the EIBnet/IP header)
     * @throws EICLException
     *             Forward the HPAI constructor Exception.
     */
    public Search_Response(byte[] _Body) throws EICLException {
        discovered_endpoint = new HPAI(_Body);
    }

    /**
     * Returns the end point encapsulated in the search response
     * 
     * @return the discovered endpoint
     */
    public HPAI getDiscoveredEndpoint() {
        return discovered_endpoint;
    }

}
