/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package tuwien.auto.eicl.struct.eibnetip.util;

import java.io.*;

import tuwien.auto.eicl.util.*;

/**
 * <p>
 * CRI_CRD = connection request information / connection response data. This
 * class is able to parse a CRI or CRD structure. All encapsulated information
 * can be accessed and set by get and set methods. The data body is restricted
 * to two bytes. (Enough for our purpose)
 * 
 * @author Bernhard Erb
 * @see tuwien.auto.eicl.struct.eibnetip.Connect_Request
 * @see tuwien.auto.eicl.struct.eibnetip.Connect_Response
 */

public class CRI_CRD {
    private short structlength;

    private short connectiontypecode;

    private short data[];

    /**
     * Creates a new instance from a byte array
     * 
     * @param _Data
     *            the byte array
     * @throws EICLException
     *             if the byte array has the wrong length this exception is
     *             thrown
     */
    public CRI_CRD(byte[] _Data) throws EICLException {
        structlength = _Data[0];
        connectiontypecode = _Data[1];
        data = new short[_Data.length - 2];
        for (int i = 2; i < _Data.length; i++) {
            data[i - 2] = _Data[i];
        }

        if (structlength != _Data.length)
            throw new EICLException("wrong CRI_CRD length");
    }

    /**
     * Creates a new instance with the given parameters.
     * 
     * @param _Connectiontypecode
     *            The connection type code.
     * @param _Data
     *            The connection type dependent data bytes.
     */
    public CRI_CRD(short _Connectiontypecode, short[] _Data) {
        structlength = new Integer(2 + _Data.length).shortValue();
        connectiontypecode = _Connectiontypecode;
        data = _Data;

    }

    /**
     * Get the structure byte array representation.
     * 
     * @return byte array The structure as byte array.
     */
    public byte[] toByteArray() {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        baos.write((byte) (structlength & 0x00FF));
        baos.write((byte) (connectiontypecode & 0x00FF));
        for (int i = 0; i < data.length; i++) {
            baos.write(data[i]);
        }
        return baos.toByteArray();
    }

    /**
     * Extracts the structure connection type code.
     * 
     * @return The structure connection type.
     */
    public short getConnectionTypeCode() {
        return connectiontypecode;
    }

    /**
     * Returns the message structure length.
     * 
     * @return The structure length
     */
    public short getStructLength() {
        return structlength;
    }

    /**
     * Get the data bytes included
     * 
     * @return The data byte array
     */
    public short[] getData() {
        return data;
    }

}
