/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package tuwien.auto.eicl.struct.eibnetip.util;

/**
 * This interface includes all Constants needed for the EIBnet/IPIP protocol
 * 
 * @author bernhard erb
 */

public interface EIBNETIP_Constants {

    /*
     * common constants
     */
    public final static short EIBNETIP_VERSION_10 = 0x10;

    public final static short HEADER_SIZE_10 = 0x06;

    /*
     * eibnetip services upper byte -> use mask for checking service type
     */
    public final static short EIBNETIP_CORE = 0x02;

    public final static short EIBNETIP_DEVMGMT = 0x03;

    public final static short EIBNETIP_TUNNELING = 0x04;

    public final static short EIBNETIP_ROUTING = 0x05;

    public final static short EIBNETIP_REMLOG = 0x06;

    public final static short EIBNETIP_REMCONF = 0x07;

    public final static short EIBNETIP_OBJSRV = 0x08;

    /*
     * core eibnetip services
     */
    public final static int SEARCH_REQUEST = 0x0201;

    public final static int SEARCH_RESPONSE = 0x0202;

    public final static int DESCRIPTION_REQUEST = 0x0203;

    public final static int DESCRIPTION_RESPONSE = 0x0204;

    public final static int CONNECT_REQUEST = 0x0205;

    public final static int CONNECT_RESPONSE = 0x0206;

    public final static int CONNECTIONSTATE_REQUEST = 0x0207;

    public final static int CONNECTIONSTATE_RESPONSE = 0x0208;

    public final static int DISCONNECT_REQUEST = 0x0209;

    public final static int DISCONNECT_RESPONSE = 0x020A;

    /*
     * device management services;
     */
    public final static int DEVICE_CONFIGURATION_REQUEST = 0x0310;

    public final static int DEVICE_CONFIGURATION_ACK = 0x0311;

    /*
     * tunnelling services
     */
    public final static int TUNNELLING_REQUEST = 0x0420;

    public final static int TUNNELLING_ACK = 0x0421;

    public final static short TUNNEL_LINKLAYER = 0x02;

    public final static short TUNNEL_RAW = 0x04;

    public final static short TUNNEL_BUSMONITOR = 0x80;

    /*
     * routing services
     */
    public final static int ROUTING_INDICATION = 0x0530;

    public final static int ROUTING_LOST_MESSAGE = 0x0531;

    /*
     * connection types
     */
    public final static short DEVICE_MGMT_CONNECTION = 0x03;

    public final static short TUNNEL_CONNECTION = 0x04;

    public final static short REMLOG_CONNECTION = 0x06;

    public final static short REMCONF_CONNECTION = 0x07;

    public final static short OBJSVR_CONNECTION = 0x08;

    /*
     * error codes common error codes
     */
    public final static short E_NO_ERROR = 0x00;

    public final static short E_HOST_PROTOCOL_TYPE = 0x01;

    public final static short E_VERSION_NOT_SUPPORTED = 0x02;

    public final static short E_SEQUENCE_NUMBER = 0x04;

    /*
     * connect response status codes
     */
    public final static short E_CONNECTION_TYPE = 0x22;

    public final static short E_CONNECTION_OPTION = 0x23;

    public final static short E_NO_MORE_CONNECTIONS = 0x24;

    /*
     * connectionstate_response status codes
     */
    public final static short E_CONNECTION_ID = 0x21;

    public final static short E_DATA_CONNECTION = 0x26;

    public final static short E_KNX_CONNECTION = 0x27;

    /*
     * tunnelling connect_ack error codes
     */
    public final static short E_TUNNELING_LAYER = 0x29;

    /**
     * device management device_configuration_ack status codes
     */
    /**
     * description information block
     */
    public final static short DEVICE_INFO = 0x01;

    public final static short SUPP_SVC_FAMILIES = 0x02;

    public final static short MFR_DATA = 0xFE;

    /**
     * medium codes
     */
    public final static short TP0 = 0x01;

    public final static short TP1 = 0x02;

    public final static short PL110 = 0x04;

    public final static short PL132 = 0x08;

    public final static short RF = 0x10;

    /**
     * host protocol codes
     */
    public final static short IPV4_UDP = 0x01;

    public final static short IPV4_TCP = 0x02;

    /**
     * timeout constants in seconds
     */
    public final static short CONNECT_REQUEST_TIMEOUT = 3;

    public final static short CONNECTIONSTATE_REQUEST_TIMEOUT = 10;

    public final static short DEVICE_CONFIGURATION_REQUEST_TIMEOUT = 10;

    public final static short HEARTBEAT_REQUEST_TIMEOUT = 60;

    /**
     * eibnet/ip object type
     */
    public final static short EIBNET_IP_OBJECT_TYPE = 13;

    /**
     * eibnet/ip property definitions
     */
    public final static short PID_PROJECT_INSTALLATION_ID = 51;

    public final static short PID_KNX_INDIVIDUAL_ADDRESSES = 52;

    public final static short PID_ADDITIONAL_INDIVIDUAL_ADDRESSES = 53;

    public final static short PID_CURRENT_IP_ASSIGNMENT_METHOD = 54;

    public final static short PID_IP_ASSIGNMENT_METHOD = 55;

    public final static short PID_IP_CAPABILITIES = 56;

    public final static short PID_CURRENT_IP_ADDRESS = 57;

    public final static short PID_CURRENT_SUBNET_MASK = 58;

    public final static short PID_CURRENT_DEFAULT_GATEWAY = 59;

    public final static short PID_IP_ADDRESS = 60;

    public final static short PID_SUBNET_MASK = 61;

    public final static short PID_DEFAULT_GATEWAY = 62;

    public final static short PID_DHCP_BOOTP_SERVER = 63;

    public final static short PID_MAC_ADDRESS = 64;

    public final static short PID_SYSTEM_SETUP_MULTICAST_ADDRESS = 65;

    public final static short PID_ROUTING_MULTICAST_ADDRESS = 66;

    public final static short PID_TTL = 67;

    public final static short PID_EIBNETIP_DEVICE_CAPABILITIES = 68;

    public final static short PID_EIBNETIP_DEVICE_STATE = 69;

    public final static short PID_EIBNETIP_ROUTING_CAPABILITIES = 70;

    public final static short PID_PRIORITY_FIFO_ENABLED = 71;

    public final static short PID_QUEUE_OVERFLOW_TO_IP = 72;

    public final static short PID_QUEUE_OVERFLOW_TO_KNX = 73;

    public final static short PID_MSG_TRANSMIT_TO_IP = 74;

    public final static short PID_MSG_TRANSMIT_TO_KNX = 75;

    public final static short PID_FRIENDLY_NAME = 76;

    /**
     * internet protocol constants
     */
    public final static int EIBNETIP_PORT_NUMBER = 3671;

    public final static String EIBNETIP_SERVER_MULTICAST = "224.0.23.12";

    public static final int EIBNETIP_CLIENT_PORT_NUMBER = 9998;

}
