/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package tuwien.auto.eicl.struct.eibnetip.util;

import java.util.*;
import java.io.*;

import tuwien.auto.eicl.util.*;

/**
 * <p>
 * This class implements the EIBnet/IP supported families description
 * information block (SUPPFAM_DIB). This structure is contained in the
 * Description response message.
 * 
 * @author Bernhard Erb
 * @see tuwien.auto.eicl.struct.eibnetip.Description_Response
 */

public class SUPPFAM_DIB {
    short structlength;

    short descriptiontypecode;

    Hashtable hash = new Hashtable();

    /**
     * Creates a new object by parsing a byte array. If something goes wrong an
     * Exception is thrown
     * 
     * @param _Body
     *            The supported family DIB as byte array.
     * @throws EICLException
     *             If the conversion does not succeed.
     */
    public SUPPFAM_DIB(byte[] _Body) throws EICLException {
        ByteArrayInputStream bais = new ByteArrayInputStream(_Body);
        structlength = (short) bais.read();
        descriptiontypecode = (short) bais.read();

        if (descriptiontypecode != EIBNETIP_Constants.SUPP_SVC_FAMILIES)
            throw new EICLException("Not a Supported families DIB");

        short servicefamily;
        short version;
        for (int i = 1; i < structlength; i = i + 2) {
            servicefamily = (short) bais.read();
            version = (short) bais.read();
            hash.put(new Integer(servicefamily), new Integer(version));
        }
    }

    /**
     * Returns an enumeration of integer values describing the supported service
     * families (e.g., Tunnelling = 4, Routing = 5, ...) Their versions can be
     * retrieved using getVersion(int family)
     * 
     * @return Supported families (as an Enumeration of Integer objects)
     */
    public Enumeration getSupportedFamilies() { // returns an enumeration with
                                                // Integer elements
        return hash.keys();
    }

    /**
     * Returns the version of a given service
     * 
     * @param _SupportedFamily
     *            the service
     * @return The version
     */
    public int getVersion(int _SupportedFamily) {
        return ((Integer) hash.get(new Integer(_SupportedFamily))).intValue();
    }

    /**
     * Returns the length of the structure.
     * 
     * @return The structure length
     */
    public short getStructlength() {
        return structlength;
    }

}
