/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package tuwien.auto.eicl.util;

import java.io.IOException;
import java.net.MulticastSocket;

import tuwien.auto.eicl.struct.eibnetip.util.EIBNETIP_Constants;

/**
 * This class contains utility methods for the Calimero API. These methods are
 * needed in multiple classes.
 * 
 * @author bernhard erb
 */

public class Util {

    private static int startport = 10001; // Starting from this number the
                                            // portnumber for subsequent sockets
                                            // are calculated (inc)

    /**
     * This method opens a new socket on a port. The portnumber is incremented
     * from the startport.
     * 
     * @return the opened multicastsocket
     */
    public static MulticastSocket openNextPort() {
        boolean notfound = true;
        ;
        MulticastSocket socket = null;

        while (notfound) {
            notfound = false;
            startport++;
            try {
                socket = new MulticastSocket(startport);
            } catch (IOException ex) {
                // The port is closed... try the next one
                notfound = true;
            }
        }
        return socket;
    }

    /**
     * Parses the error strings and returns a huam readable string
     * representation
     * 
     * @param _Status
     *            the error status
     * @return A human readable error string.
     */
    public static String getStatusString(short _Status) {
        switch (_Status) {
        case EIBNETIP_Constants.E_CONNECTION_ID:
            return "Wrong channelid";
        case EIBNETIP_Constants.E_CONNECTION_OPTION:
            return "Connection option not supported";
        case EIBNETIP_Constants.E_CONNECTION_TYPE:
            return "Connection type not supported";
        case EIBNETIP_Constants.E_DATA_CONNECTION:
            return "Problem with data connection";
        case EIBNETIP_Constants.E_HOST_PROTOCOL_TYPE:
            return "Hostprotocol type not supported";
        case EIBNETIP_Constants.E_KNX_CONNECTION:
            return "KNX connection error";
        case EIBNETIP_Constants.E_NO_ERROR:
            return "No error";
        case EIBNETIP_Constants.E_NO_MORE_CONNECTIONS:
            return "No more connections";
        case EIBNETIP_Constants.E_SEQUENCE_NUMBER:
            return "Wrong Sequencenumber";
        case EIBNETIP_Constants.E_TUNNELING_LAYER:
            return "Tunneling problem";
        case EIBNETIP_Constants.E_VERSION_NOT_SUPPORTED:
            return "EIBNETIP version not supported";
        default:
            return "Unknown error";
        }

    }
}
