/*
 * Copyright (C) 2005 Bernhard Erb
 
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package tuwien.auto.eicldemo.ui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

import tuwien.auto.eicl.*;
import tuwien.auto.eicl.struct.eibnetip.Description_Response;
import tuwien.auto.eicl.struct.eibnetip.util.HPAI;

import java.net.*;

/**
 * <p>
 * Title:
 * </p>
 * <p>
 * Description:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * <p>
 * Company:
 * </p>
 * 
 * @author not attributable
 * @version 1.0
 */

public class Dialog_Connect extends JDialog {
    public static final long serialVersionUID = 1;

    private JButton b_Cancel = new JButton();

    private JButton b_Search = new JButton();

    private JButton b_Description = new JButton();

    private JComboBox cb_Host = new JComboBox();

    private JButton b_Connect = new JButton();

    private JLabel l_host = new JLabel();

    private JLabel l_Port = new JLabel();

    private JTextField tfPort = new JTextField();

    private JLabel l_LocalIP = new JLabel();

    private JTextField t_LocalIP = new JTextField();

    private Discoverer factory;

    public CEMI_Connection tunnel;

    public Dialog_Connect(Main_Frame frame, String title, boolean modal, Discoverer _Factory) {
        super(frame, title, modal);
        factory = _Factory;
        try {
            init();
        } catch (Exception ex) {
            Main_Frame.showException(ex.getMessage());
        }
    }

    public Dialog_Connect(Discoverer _Factory) {
        this(null, "", false, _Factory);
    }

    private void init() throws Exception {
        this.getContentPane().setLayout(null);
        this.setSize(311 + 20, 176 + 32);
        l_host.setText("Host");
        l_host.setBounds(new Rectangle(12, 10, 40, 20));
        cb_Host.setEditable(true);
        cb_Host.setToolTipText("example 192.168.0.1");
        cb_Host.setBounds(60 + 20, 10, 228, 20);

        l_Port.setText("Port");
        l_Port.setBounds(new Rectangle(12, 40, 40, 20));
        tfPort.setText("3671");
        tfPort.setBounds(60 + 20, 40, 100, 20);

        l_LocalIP.setText("Local IP");
        l_LocalIP.setBounds(new Rectangle(12, 70, 60, 20));
        t_LocalIP.setText(tuwien.auto.eicl.struct.eibnetip.util.HPAI.getLocalIP().getHostAddress());
        t_LocalIP.setBounds(new Rectangle(60 + 20, 70, 120, 20));

        b_Description.setBounds(12, 72 + 32, 160, 20);
        b_Description.setText("Request description");
        b_Description.addActionListener(new DialogConnect_Description_actionAdapter(this));
        b_Connect.setBounds(190, 72 + 32, 100, 20);
        b_Connect.setText("Connect");
        b_Connect.addActionListener(new DialogConnect_Connect_actionAdapter(this));

        b_Search.setBounds(12, 117 + 32, 157, 20);
        b_Search.setText("Search for servers");
        b_Search.addActionListener(new DialogConnect_Search_actionAdapter(this));
        b_Cancel.setBounds(190, 117 + 32, 100, 20);
        b_Cancel.setText("Cancel");
        b_Cancel.addActionListener(new DialogConnect_Cancel_actionAdapter(this));
        this.getContentPane().add(l_host, null);
        this.getContentPane().add(cb_Host, null);
        this.getContentPane().add(b_Cancel, null);
        this.getContentPane().add(b_Search, null);
        this.getContentPane().add(b_Connect, null);
        this.getContentPane().add(b_Description, null);
        this.getContentPane().add(l_Port, null);
        this.getContentPane().add(tfPort, null);
        this.getContentPane().add(l_LocalIP, null);
        this.getContentPane().add(t_LocalIP, null);
    }

    public void Search_actionPerformed(ActionEvent e) {
        try {
            HPAI.setLocalIP(InetAddress.getByName(t_LocalIP.getText()));
            cb_Host.removeAllItems();
            cb_Host.setToolTipText("");
            factory.sendSearchRequest();
            synchronized (this) {
                this.wait(2000);
            }
            InetSocketAddress[] isa = factory.getSearchResponses();
            if (isa.length > 0) {
                tfPort.setEditable(false);
                cb_Host.setEditable(false);
                for (int i = 0; i < isa.length; i++) {
                    cb_Host.addItem(isa[i].getHostName());
                }
                cb_Host.setSelectedIndex(0);
            }
        } catch (Exception ex) {
            Main_Frame.showException(ex.getMessage());
        }
    }

    public void Connect_actionPerformed(ActionEvent e) {
        try {
            HPAI.setLocalIP(InetAddress.getByName(t_LocalIP.getText()));

            if (tfPort.isEditable()) {
                InetSocketAddress isa = new InetSocketAddress(cb_Host.getSelectedItem().toString(),
                        Integer.valueOf(tfPort.getText()).intValue());
                tunnel = new CEMI_Connection(isa, new TunnellingConnectionType());
            } else {
                tunnel = new CEMI_Connection(
                        factory.getSearchResponses()[cb_Host.getSelectedIndex()],
                        new TunnellingConnectionType());
            }
            this.dispose();
        } catch (Exception ex) {
            Main_Frame.showException(ex.getMessage());
        }

    }

    public void Description_actionPerformed(ActionEvent e) {
        InetSocketAddress isa = null;
        try {
            HPAI.setLocalIP(InetAddress.getByName(t_LocalIP.getText()));

            if (tfPort.isEditable()) {
                isa = new InetSocketAddress(cb_Host.getSelectedItem().toString(), Integer.valueOf(
                        tfPort.getText()).intValue());
            } else {
                isa = factory.getSearchResponses()[cb_Host.getSelectedIndex()];
            }
            try {
                Description_Response response = factory.getDescription(isa, 3);
                synchronized (Main_Frame.textArea) {
                    Main_Frame.textArea.append("Description response received from " + isa + "\n");
                    Main_Frame.textArea.setCaretPosition(Main_Frame.textArea.getText().length());
                    Main_Frame.textArea.append(response.getDevice().getName() + "\n");
                    Main_Frame.textArea.setCaretPosition(Main_Frame.textArea.getText().length());
                    Main_Frame.textArea.append("Supports ");
                    for (java.util.Enumeration sf = response.getSupportedFamilies().getSupportedFamilies(); sf.hasMoreElements();) {
                        switch (((Integer) sf.nextElement()).intValue()) {
                        case 2:
                            Main_Frame.textArea.append("Core ");
                            break;
                        case 3:
                            Main_Frame.textArea.append("DeviceMgmt ");
                            break;
                        case 4:
                            Main_Frame.textArea.append("Tunnelling ");
                            break;
                        case 5:
                            Main_Frame.textArea.append("Routing ");
                            break;
                        }
                    }
                    Main_Frame.textArea.append("\n");
                    Main_Frame.textArea.setCaretPosition(Main_Frame.textArea.getText().length());
                    Main_Frame.textArea.append("EIB Address "
                            + response.getDevice().getEibaddress().toString() + "\n\n");
                    Main_Frame.textArea.setCaretPosition(Main_Frame.textArea.getText().length());
                }
            }

            catch (Exception ex) {
                Main_Frame.showException(ex.getMessage());
            }
        } catch (UnknownHostException ex) {
            Main_Frame.showException(ex.getMessage());

        }
    }

}

class DialogConnect_Cancel_actionAdapter implements ActionListener {
    Dialog_Connect conni;

    DialogConnect_Cancel_actionAdapter(Dialog_Connect adaptee) {
        conni = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        conni.dispose();
    }
}

class DialogConnect_Search_actionAdapter implements ActionListener {
    Dialog_Connect conni;

    DialogConnect_Search_actionAdapter(Dialog_Connect adaptee) {
        conni = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        conni.Search_actionPerformed(e);
    }
}

class DialogConnect_Connect_actionAdapter implements ActionListener {
    Dialog_Connect conni;

    DialogConnect_Connect_actionAdapter(Dialog_Connect adaptee) {
        conni = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        conni.Connect_actionPerformed(e);
    }
}

class DialogConnect_Description_actionAdapter implements ActionListener {
    Dialog_Connect conni;

    DialogConnect_Description_actionAdapter(Dialog_Connect adaptee) {
        conni = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        conni.Description_actionPerformed(e);
    }
}
