/*
 * Copyright (C) 2005 Bernhard Erb and contributors

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/*
 $Log: Dialog_Read.java,v $
 Revision 1.6  2005/12/28 09:59:21  gneug
 xlator lookup fixed

 */

package tuwien.auto.eicldemo.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JFrame;

import tuwien.auto.eibpoints.*;
import tuwien.auto.eibxlator.PointPDUXlator;
import tuwien.auto.eibxlator.PDUXlatorList;
import tuwien.auto.eicl.CEMI_Connection;
import tuwien.auto.eicl.struct.cemi.CEMI_L_DATA;
import tuwien.auto.eicl.util.*;
import tuwien.auto.eicl.struct.cemi.EIB_Address;

public class Dialog_Read extends javax.swing.JDialog {
    public static final long serialVersionUID = 1;

    private JComboBox cb_Points;

    private JButton b_Cancel;

    private JButton b_Read;

    private PointList pointList;

    private CEMI_Connection tunnel;

    public Dialog_Read(JFrame frame, PointList _PointList, CEMI_Connection _Tunnel) {
        super(frame);
        pointList = _PointList;
        tunnel = _Tunnel;
        initGUI();

    }

    private void initGUI() {
        this.getContentPane().setLayout(null);
        {
            cb_Points = new JComboBox();
            Point[] p = pointList.getAllPoints();
            for (int i = 0; i < p.length; i++) {
                cb_Points.addItem(p[i].getDeviceName());
            }
            this.getContentPane().add(cb_Points);
            cb_Points.setBounds(30, 10, 160, 20);
        }
        {
            b_Read = new JButton();
            this.getContentPane().add(b_Read);
            b_Read.setText("Read");
            b_Read.setBounds(10, 50, 100, 20);
            b_Read.addActionListener(new DialogRead_Read_actionAdapter(this));
        }
        {
            b_Cancel = new JButton();
            this.getContentPane().add(b_Cancel);
            b_Cancel.setText("Cancel");
            b_Cancel.setBounds(120, 50, 100, 20);
            b_Cancel.addActionListener(new DialogRead_Cancel_actionAdapter(this));
        }
        try {
            setSize(240, 110);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void Cancel_actionPerformed(ActionEvent e) {
        this.dispose();
    }

    public void Read_actionPerformed(ActionEvent e) {
        try {
            Point p = pointList.getPoint(cb_Points.getSelectedItem().toString());
            PointPDUXlator xlator = PDUXlatorList.getPointPDUXlator(p.getMajorType(),
                    p.getMinorType());
            xlator.setServiceType(PointPDUXlator.A_GROUPVALUE_READ);
            CEMI_L_DATA data = new CEMI_L_DATA((byte) CEMI_L_DATA.MC_L_DATAREQ, new EIB_Address(),
                    p.getDeviceAddress()[0], xlator.getAPDUByteArray());
            tunnel.sendFrame(data, CEMI_Connection.WAIT_FOR_CONFIRM);
            synchronized (Main_Frame.textArea) {
                Main_Frame.textArea.append("Read request to " + p.getDeviceName() + " sent \n");
            }
            this.dispose();
        } catch (EICLException ex) {
            Main_Frame.showException(ex.getMessage());
        }
    }

}

class DialogRead_Cancel_actionAdapter implements ActionListener {
    Dialog_Read conni;

    public DialogRead_Cancel_actionAdapter(Dialog_Read adaptee) {
        conni = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        conni.Cancel_actionPerformed(e);
    }
}

class DialogRead_Read_actionAdapter implements ActionListener {
    Dialog_Read conni;

    public DialogRead_Read_actionAdapter(Dialog_Read adaptee) {
        conni = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        conni.Read_actionPerformed(e);
    }
}