/*
 * Copyright (C) 2005 Bernhard Erb

 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.

 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.

 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package tuwien.auto.eicldemo.ui;

import java.awt.AWTEvent;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.FileDialog;
import java.awt.event.*;
import java.net.*;
import javax.swing.*;
import tuwien.auto.eibpoints.*;
import tuwien.auto.eicl.*;
import tuwien.auto.eicl.event.*;
import tuwien.auto.eicl.struct.cemi.CEMI_L_DATA;
import tuwien.auto.eicl.util.*;
import tuwien.auto.eibxlator.*;

/**
 * <p>
 * Title:
 * </p>
 * <p>
 * Description:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2005
 * </p>
 * <p>
 * Company:
 * </p>
 * 
 * @author not attributable
 * @version 1.0
 */

public class Main_Frame extends JFrame {
    private JPanel contentPane;

    private JMenuBar MenuBar = new JMenuBar();

    private JMenu M_File = new JMenu();

    private JMenu M_PointList = new JMenu();

    private JMenu M_ReadWrite = new JMenu();

    private JMenuItem MI_Connect = new JMenuItem();

    private JMenuItem MI_Send = new JMenuItem();

    private JMenuItem MI_Read = new JMenuItem();

    private JMenuItem MI_Disconnect = new JMenuItem();

    private JMenuItem MI_Search = new JMenuItem();

    private JMenuItem MI_PointList = new JMenuItem();

    private JMenuItem MI_Open = new JMenuItem();

    private JMenuItem MI_Save = new JMenuItem();

    private JMenuItem MI_Quit = new JMenuItem();

    private JMenu M_Help = new JMenu();

    private JMenuItem MI_About = new JMenuItem();

    public static JTextArea textArea = new JTextArea();

    private JScrollPane scrollText = new JScrollPane(textArea);

    public static final long serialVersionUID = 1;

    //
    private Discoverer factory;

    private CEMI_Connection tunnel = null;

    private PointList pointList = new PointList();

    // Construct the frame
    public Main_Frame() {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // Component initialization
    private void jbInit() throws Exception {
        contentPane = (JPanel) this.getContentPane();
        contentPane.setLayout(null);
        this.setSize(new Dimension(620, 500));
        this.setTitle("Calimero DEMO / EICL");
        M_File.setText("File");
        M_PointList.setText("List");
        M_ReadWrite.setText("Read/Write");
        MI_Search.setText("Search");
        MI_Search.addActionListener(new MainFrame_MI_Search_actionAdapter(this));
        MI_Connect.setText("Connect");
        MI_Connect.addActionListener(new MainFrame_MI_Connect_actionAdapter(this));
        MI_Send.setText("Write");
        MI_Send.addActionListener(new MainFrame_MI_Send_actionAdapter(this));
        MI_Read.setText("Read");
        MI_Read.addActionListener(new MainFrame_MI_Read_actionAdapter(this));

        MI_Disconnect.setText("Disconnect");
        MI_Disconnect.addActionListener(new MainFrame_MI_Disconnect_actionAdapter(this));
        MI_Open.setText("Open");
        MI_Open.addActionListener(new MainFrame_MI_Open_actionAdapter(this));
        MI_PointList.setText("Adm. Point List");
        MI_PointList.addActionListener(new MainFrame_MI_PointList_actionAdapter(this));
        MI_Save.setText("Save");
        MI_Save.addActionListener(new MainFrame_MI_Save_actionAdapter(this));
        MI_Quit.setText("Quit");
        MI_Quit.addActionListener(new MainFrame_MI_Quit_actionAdapter(this));
        M_Help.setText("Help");
        MI_About.setText("About");
        MI_About.addActionListener(new MainFrame_MI_About_actionAdapter(this));

        textArea.setEditable(false);
        scrollText.setBounds(new Rectangle(10, 40, 590, 400));
        scrollText.setAutoscrolls(true);

        MenuBar.setBounds(new Rectangle(0, 0, 608, 25));
        contentPane.add(scrollText, null);
        MenuBar.add(M_File);
        MenuBar.add(M_PointList);
        MenuBar.add(M_ReadWrite);
        MenuBar.add(M_Help);
        M_PointList.add(MI_Open);
        M_PointList.add(MI_PointList);
        M_PointList.add(MI_Save);
        M_File.add(MI_Connect);
        M_File.add(MI_Send);
        M_File.add(MI_Disconnect);
        M_File.addSeparator();
        M_File.add(MI_Quit);
        M_ReadWrite.add(MI_Send);
        M_ReadWrite.add(MI_Read);
        M_Help.add(MI_About);
        contentPane.add(MenuBar, null);

        factory = new Discoverer();

    }

    // Overridden so we can exit when window is closed
    protected void processWindowEvent(WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
            shutdown();
        }
    }

    static void showException(String _Message) {
        MessageDialog md = new MessageDialog(_Message, MessageDialog.ERROR_PIC);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = md.getSize();
        if (frameSize.height > screenSize.height) {
            frameSize.height = screenSize.height;
        }
        if (frameSize.width > screenSize.width) {
            frameSize.width = screenSize.width;
        }
        md.setLocation((screenSize.width - frameSize.width) / 2,
                (screenSize.height - frameSize.height) / 2);
        md.setVisible(true);
    }

    void MI_Search_actionPerformed(ActionEvent e) {
        try {
            factory.sendSearchRequest();
            synchronized (this) {
                wait(1000);
            }
            InetSocketAddress[] sa = factory.getSearchResponses();
            for (int i = 0; i < sa.length; i++) {
                synchronized (textArea) {
                    textArea.append("Search response from host " + sa[i].getAddress() + " "
                            + sa[i].getPort() + "\n");
                    textArea.setCaretPosition(textArea.getText().length());
                }
            }
        } catch (Exception ex) {
            showException(ex.getMessage());
        }
    }

    void MI_Connect_actionPerformed(ActionEvent e) {
        Dialog_Connect conni = new Dialog_Connect(this, "Connect to", true, factory);

        // Center the window
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = conni.getSize();
        if (frameSize.height > screenSize.height) {
            frameSize.height = screenSize.height;
        }
        if (frameSize.width > screenSize.width) {
            frameSize.width = screenSize.width;
        }
        conni.setLocation((screenSize.width - frameSize.width) / 2,
                (screenSize.height - frameSize.height) / 2);
        conni.setVisible(true);

        if (conni.tunnel != null) {
            tunnel = conni.tunnel;
            tunnel.addFrameListener(new My_EICLEventListener(this));

            synchronized (textArea) {
                textArea.append("Client connected \n");
                textArea.setCaretPosition(textArea.getText().length());
            }

        }
    }

    void MI_Disconnect_actionPerformed(ActionEvent e) {
        try {
            if (tunnel != null)
                tunnel.disconnect("client request");
        } catch (EICLException ex) {
            showException(ex.getMessage());
        }
    }

    void MI_Quit_actionPerformed(ActionEvent e) {
        shutdown();
    }

    private void shutdown() {
        try {
            if (tunnel != null)
                tunnel.disconnect("client request");
        } catch (EICLException ex) {
            showException(ex.getMessage());
        }
        this.dispose();
        System.exit(0);
    }

    void MI_PointList_actionPerformed(ActionEvent e) {
        Dialog_Point point = new Dialog_Point(this, pointList);

        // Center the window
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = point.getSize();
        if (frameSize.height > screenSize.height) {
            frameSize.height = screenSize.height;
        }
        if (frameSize.width > screenSize.width) {
            frameSize.width = screenSize.width;
        }
        point.setLocation((screenSize.width - frameSize.width) / 2,
                (screenSize.height - frameSize.height) / 2);
        point.setVisible(true);

    }

    void MI_Save_actionPerformed(ActionEvent e) {
        FileDialog fd = new FileDialog(this, "Save Point List", FileDialog.SAVE);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = fd.getSize();
        if (frameSize.height > screenSize.height) {
            frameSize.height = screenSize.height;
        }
        if (frameSize.width > screenSize.width) {
            frameSize.width = screenSize.width;
        }
        fd.setLocation((screenSize.width - frameSize.width) / 2,
                (screenSize.height - frameSize.height) / 2);
        fd.setVisible(true);

        try {
            pointList.toFile(fd.getDirectory() + "/" + fd.getFile());
        } catch (EICLException ex) {
            showException(ex.getMessage());
        }
    }

    void MI_Open_actionPerformed(ActionEvent e) {
        FileDialog fd = new FileDialog(this, "Open Point List", FileDialog.LOAD);
        fd.setVisible(true);

        if (fd.getFile() != null) {
            try {
                pointList = new PointList(fd.getDirectory() + fd.getFile());
            } catch (EICLException ex) {
                showException(ex.getMessage());
            }
        }
    }

    void MI_About_actionPerformed(ActionEvent e) {
        MessageDialog md = new MessageDialog(
                "Department of Automation\nTU Wien\n\nBernhard Erb and contributors\n\nhttp://calimero.sf.net",
                MessageDialog.INFO_PIC);

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = md.getSize();
        if (frameSize.height > screenSize.height) {
            frameSize.height = screenSize.height;
        }
        if (frameSize.width > screenSize.width) {
            frameSize.width = screenSize.width;
        }
        md.setLocation((screenSize.width - frameSize.width) / 2,
                (screenSize.height - frameSize.height) / 2);
        md.setVisible(true);
    }

    void packetReceived(FrameEvent e) {
        CEMI_L_DATA data = (CEMI_L_DATA) e.getPacket();

        String from = "";
        try {
            from = pointList.getPoint(data.getDestinationAddress()).getDeviceName();
        } catch (EICLException ex) {
            from = data.getDestinationAddress().toString();
        }
        synchronized (textArea) {
            textArea.append("packet received from " + data.getSourceAddress().toString() + " to "
                    + from + " value ");
            textArea.setCaretPosition(textArea.getText().length());
            try {
                Point p = pointList.getPoint(data.getDestinationAddress());
                PointPDUXlator ppx = PDUXlatorList.getPointPDUXlator(p.getMajorType(),
                        p.getMinorType());
                ppx.setAPDUByteArray(data.getData());
                textArea.append(" " + ppx.getASDUasString());
            } catch (EICLException ex) {
                byte[] d = data.getData();
                for (int i = 0; i < d.length; i++) {
                    short help = d[i];
                    textArea.append((help & 0x00FF) + " ");
                }
            }
            textArea.append("\n");
        }
    }

    void TunnelServerError(DisconnectEvent _DisconnectEvent) {
        synchronized (textArea) {
            textArea.append("Client disconnected: " + _DisconnectEvent.getDisconnectMessage()
                    + "\n");
            textArea.setCaretPosition(textArea.getText().length());
        }
        tunnel = null;
    }

    void MI_Send_actionPerformed(ActionEvent e) {
        if (tunnel != null && pointList.getAllPoints().length > 0) {
            Dialog_Send send = new Dialog_Send(this, pointList, tunnel);

            // Center the window
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            Dimension frameSize = send.getSize();
            if (frameSize.height > screenSize.height) {
                frameSize.height = screenSize.height;
            }
            if (frameSize.width > screenSize.width) {
                frameSize.width = screenSize.width;
            }
            send.setLocation((screenSize.width - frameSize.width) / 2,
                    (screenSize.height - frameSize.height) / 2);
            send.setVisible(true);
        }

    }

    void MI_Read_actionPerformed(ActionEvent e) {
        if (tunnel != null && pointList.getAllPoints().length > 0) {
            Dialog_Read send = new Dialog_Read(this, pointList, tunnel);

            // Center the window
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            Dimension frameSize = send.getSize();
            if (frameSize.height > screenSize.height) {
                frameSize.height = screenSize.height;
            }
            if (frameSize.width > screenSize.width) {
                frameSize.width = screenSize.width;
            }
            send.setLocation((screenSize.width - frameSize.width) / 2,
                    (screenSize.height - frameSize.height) / 2);
            send.setVisible(true);
        }

    }

}

class MainFrame_MI_Search_actionAdapter implements ActionListener {
    Main_Frame adaptee;

    MainFrame_MI_Search_actionAdapter(Main_Frame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.MI_Search_actionPerformed(e);
    }
}

class MainFrame_MI_Connect_actionAdapter implements ActionListener {
    Main_Frame adaptee;

    MainFrame_MI_Connect_actionAdapter(Main_Frame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.MI_Connect_actionPerformed(e);
    }
}

class MainFrame_MI_Disconnect_actionAdapter implements ActionListener {
    Main_Frame adaptee;

    MainFrame_MI_Disconnect_actionAdapter(Main_Frame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.MI_Disconnect_actionPerformed(e);
    }
}

class MainFrame_MI_Quit_actionAdapter implements ActionListener {
    Main_Frame adaptee;

    MainFrame_MI_Quit_actionAdapter(Main_Frame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.MI_Quit_actionPerformed(e);
    }
}

class MainFrame_MI_About_actionAdapter implements ActionListener {
    Main_Frame adaptee;

    MainFrame_MI_About_actionAdapter(Main_Frame adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.MI_About_actionPerformed(e);
    }
}

class MainFrame_MI_Send_actionAdapter implements ActionListener {
    Main_Frame main;

    MainFrame_MI_Send_actionAdapter(Main_Frame _MainFrame) {
        main = _MainFrame;
    }

    public void actionPerformed(ActionEvent e) {
        main.MI_Send_actionPerformed(e);
    }
}

class MainFrame_MI_PointList_actionAdapter implements ActionListener {
    Main_Frame main;

    MainFrame_MI_PointList_actionAdapter(Main_Frame _MainFrame) {
        main = _MainFrame;
    }

    public void actionPerformed(ActionEvent e) {
        main.MI_PointList_actionPerformed(e);
    }
}

class MainFrame_MI_Open_actionAdapter implements ActionListener {
    Main_Frame main;

    MainFrame_MI_Open_actionAdapter(Main_Frame _MainFrame) {
        main = _MainFrame;
    }

    public void actionPerformed(ActionEvent e) {
        main.MI_Open_actionPerformed(e);
    }
}

class MainFrame_MI_Save_actionAdapter implements ActionListener {
    Main_Frame main;

    MainFrame_MI_Save_actionAdapter(Main_Frame _MainFrame) {
        main = _MainFrame;
    }

    public void actionPerformed(ActionEvent e) {
        main.MI_Save_actionPerformed(e);
    }
}

class MainFrame_MI_Read_actionAdapter implements ActionListener {
    Main_Frame main;

    MainFrame_MI_Read_actionAdapter(Main_Frame _MainFrame) {
        main = _MainFrame;
    }

    public void actionPerformed(ActionEvent e) {
        main.MI_Read_actionPerformed(e);
    }
}

class My_EICLEventListener implements EICLEventListener {
    Main_Frame main;

    My_EICLEventListener(Main_Frame adaptee) {
        main = adaptee;
    }

    public void newFrameReceived(FrameEvent e) {
        main.packetReceived(e);
    }

    public void serverDisconnected(DisconnectEvent e) {
        main.TunnelServerError(e);
    }

}
